/*
 * \file IMP/container/container_config.h
 * \brief Provide macros to mark functions and classes as exported
 *        from a DLL/.so, and to set up namespaces
 *
 * When building the module, IMPCONTAINER_EXPORTS should be defined, and when
 * using the module externally, it should not be. Classes and functions
 * defined in the module's headers should then be marked with
 * IMPCONTAINEREXPORT if they are intended to be part of the API, or with
 * IMPCONTAINERLOCAL if they are not (the latter is the default).
 *
 * The Windows build environment requires applications to mark exports in
 * this way; we use the same markings to set the visibility of ELF symbols
 * if we have compiler support.
 *
 * All code in this module should live in the IMP::container namespace.
 * This is simply achieved by wrapping things with the
 * IMPCONTAINER_BEGIN_NAMESPACE and IMPCONTAINER_END_NAMESPACE macros.
 * There are similar macros for module code that is designed to be for
 * internal use only.
 *
 * This header is auto-generated by tools/imp-module.py; it should not be
 * edited manually.
 *
 * Copyright 2007-2010 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPCONTAINER_CONFIG_H
#define IMPCONTAINER_CONFIG_H

#include <boost/static_assert.hpp>

#ifdef _MSC_VER
#ifdef IMPCONTAINER_EXPORTS
#define IMPCONTAINEREXPORT __declspec(dllexport)
#else
#define IMPCONTAINEREXPORT __declspec(dllimport)
#endif
#define IMPCONTAINERLOCAL
#else
#ifdef GCC_VISIBILITY
#define IMPCONTAINEREXPORT __attribute__ ((visibility("default")))
#define IMPCONTAINERLOCAL __attribute__ ((visibility("hidden")))
#else
#define IMPCONTAINEREXPORT
#define IMPCONTAINERLOCAL
#endif
#endif

#if _MSC_VER && !defined(SWIG) && !defined(IMP_DOXYGEN)
#ifdef IMPCONTAINER_EXPORTS

#define IMPCONTAINER_EXPORT_TEMPLATE(name)       template class __declspec(dllexport) name

#else

#define IMPCONTAINER_EXPORT_TEMPLATE(name)       template class __declspec(dllimport) name

#endif

#else
#define IMPCONTAINER_EXPORT_TEMPLATE(name)

#endif



#define IMPCONTAINER_BEGIN_NAMESPACE \
namespace IMP {\
namespace container {\

#define IMPCONTAINER_END_NAMESPACE \
} /* namespace IMP */ \
} /* namespace container */ \

#define IMPCONTAINER_BEGIN_INTERNAL_NAMESPACE \
IMPCONTAINER_BEGIN_NAMESPACE \
namespace internal {


#define IMPCONTAINER_END_INTERNAL_NAMESPACE \
} /* namespace internal */ \
IMPCONTAINER_END_NAMESPACE




//  functions are defined explicitly for swig
#ifndef SWIG
namespace IMP {
  class VersionInfo;
}

#include <IMP/internal/directories.h>
#include <IMP/kernel_config.h>
#include <string>

IMP_BEGIN_INTERNAL_NAMESPACE
IMPEXPORT std::string get_data_path(std::string module_name,
                                    std::string file_name);
IMPEXPORT std::string get_example_path(std::string module_name,
                                       std::string file_name);

IMP_END_INTERNAL_NAMESPACE

IMPCONTAINER_BEGIN_NAMESPACE
IMPCONTAINEREXPORT const VersionInfo& get_module_version_info();

inline std::string get_module_name() {
   return "IMP::container";
}

//! Return the path to installed data for this module
/** Each module has its own data directory, so be sure to use
    the version of this function in the correct module. To read
    the data file "data_library" that was placed in the \c data
    directory of module "mymodule", do something like
    \code
    std::ifstream in(IMP::mymodule::get_data_path("data_library"));
    \endcode
    This will ensure that the code works when \imp is installed or
    used via the \c tools/imppy.sh script.
*/
inline std::string get_data_path(std::string file_name) {
  return IMP::internal::get_data_path("container", file_name);
}

//! Return the path to installed example data for this module
/** Each module has its own example directory, so be sure to use
    the version of this function in the correct module.  For example
    to read the file \c example_protein.pdb located in the
    \c examples directory of the IMP::atom module, do
    \code
    IMP::atom::read_pdb(IMP::atom::get_example_path("example_protein.pdb", model));
    \endcode
    This will ensure that the code works when \imp is installed or
    used via the \c tools/imppy.sh script.
*/
inline std::string get_example_path(std::string file_name)  {
  return IMP::internal::get_example_path("container", file_name);
}


IMPCONTAINER_END_NAMESPACE

#endif // SWIG

#endif  /* IMPCONTAINER_CONFIG_H */
